@@ -26,7 +26,13 @@ class Instance(object):
         except AttributeError:
             return False
 
-    def get_attribute(self, name: str, **kwargs):
+    def get_attribute(self, name: str, *arg, **kwargs):
+        def raise_exception(name: str, value):
+            raise AttributeError(f"The attribute '{name}' does not exist.")
+
+        def default_value(name, value):
+            return value
+
         def get_attribute_from_dictionary(name: str, dictionary: dict):
             sections = name.split(".")
 
@@ -49,6 +55,16 @@ class Instance(object):
 
             raise AttributeError()
 
+        value = None
+        attribute_doesnt_exist = raise_exception
+        if "default" in kwargs:
+            attribute_doesnt_exist = default_value
+            value = kwargs["default"]
+
+        elif len(arg) == 1:
+            attribute_doesnt_exist = default_value
+            value = arg[0]
+
         if name == "prediction":
             return self.prediction
 
@@ -58,18 +74,13 @@ class Instance(object):
         if name == "image":
             return self.image
 
-        _error_message = f"The attribute '{name}' does not exist."
-
         if name.startswith("metadata."):
             try:
                 return get_attribute_from_dictionary(
                     name[len("metadata.") :], self.metadata
                 )
             except AttributeError:
-                if "default" in kwargs:
-                    return kwargs["default"]
-
-                raise AttributeError(_error_message)
+                return attribute_doesnt_exist(name, value)
 
         if name.startswith("properties."):
             try:
@@ -77,15 +88,9 @@ class Instance(object):
                     name[len("properties.") :], self.properties
                 )
             except AttributeError:
-                if "default" in kwargs:
-                    return kwargs["default"]
+                return attribute_doesnt_exist(name, value)
 
-                raise AttributeError(_error_message)
-
-        if "default" in kwargs:
-            return kwargs["default"]
-
-        raise AttributeError(_error_message)
+        return attribute_doesnt_exist(name, value)
 
     @staticmethod
     def create(data):
