@@ -14,6 +14,66 @@ from botocore.exceptions import ClientError, EndpointConnectionError
 PIPELINE_MODULE_NAME = "alira.modules.s3"
 
 
+def aws_s3_upload(
+    aws_s3_bucket,
+    aws_s3_public,
+    s3_key,
+    filename,
+    aws_access_key,
+    aws_secret_key,
+    aws_region_name,
+):
+    logging.info(
+        f'Uploading {filename} to bucket "{aws_s3_bucket}" and location {s3_key}...'
+    )
+    try:
+        session = boto3.Session(
+            aws_access_key_id=aws_access_key,
+            aws_secret_access_key=aws_secret_key,
+            region_name=aws_region_name,
+        )
+        client = session.client("s3")
+
+        with open(filename, "rb") as file:
+            buffer = file.read()
+
+        arguments = {"Bucket": aws_s3_bucket, "Key": s3_key, "Body": buffer}
+        content_type = "binary/octet-stream"
+
+        extension = Path(filename).suffix.lower()
+
+        if extension in ["jpg", "jpeg"]:
+            content_type = "image/jpeg"
+        elif extension == "png":
+            content_type = "image/png"
+
+        arguments["ContentType"] = content_type
+
+        if aws_s3_public:
+            arguments["ACL"] = "public-read"
+
+        client.put_object(**arguments)
+
+        s3_location = f"s3://{aws_s3_bucket}/{s3_key}"
+        logging.info(f"Uploaded {filename} to {s3_location}")
+
+        return s3_location
+
+    except EndpointConnectionError as e:
+        logging.exception(e)
+
+        # If we can't connect to the service to upload the image, let's
+        # raise a RuntimeError so that the process can be retried as part
+        # of the Redis queue.
+        raise RuntimeError(e)
+    except ClientError as e:
+        logging.exception(e)
+    except Exception as e:
+        logging.exception(e)
+
+    return None
+
+
 class S3(RedisModule):
     def __init__(
         self,
@@ -25,8 +85,12 @@ class S3(RedisModule):
         aws_access_key: str,
         aws_secret_key: str,
         aws_region_name: str,
+        id: str = None,
+        autogenerate_name: bool = False,
         redis_server: str = None,
         filtering: str = None,
+        file: str = None,
+        aws_s3_upload_function: callable = aws_s3_upload,
         **kwargs,
     ):
         super().__init__(
@@ -44,11 +108,17 @@ class S3(RedisModule):
         self.aws_s3_bucket = aws_s3_bucket
         self.aws_s3_key_prefix = aws_s3_key_prefix
 
+        self.autogenerate_name = autogenerate_name
+
         if self.aws_s3_key_prefix and self.aws_s3_key_prefix.endswith("/"):
             self.aws_s3_key_prefix = self.aws_s3_key_prefix[:-1]
 
         self.aws_s3_public = aws_s3_public
 
+        self.id = id
+        self.file = file
+        self.aws_s3_upload_function = aws_s3_upload_function
+
     def run(self, instance: Instance, **kwargs):
         if self.filtering and not self.filtering(instance):
             logging.info(
@@ -60,14 +130,23 @@ class S3(RedisModule):
             logging.info("The instance doesn't have an image")
             return None
 
-        image_filename = os.path.join(
+        if self.file:
+            filename = instance.get_attribute(self.file, default=instance.image)
+        else:
+            filename = instance.image
+
+        filename = os.path.join(
             self.configuration_directory,
             self.model_identifier,
             "images",
-            instance.image,
+            filename,
         )
 
-        s3_key = f"{instance.image}-{uuid.uuid4().hex}.png"
+        if self.autogenerate_name:
+            _, file_extension = os.path.splitext(filename)
+            s3_key = f"{uuid.uuid4().hex}.{file_extension}"
+        else:
+            s3_key = filename
 
         if self.aws_s3_key_prefix:
             s3_key = f"{self.aws_s3_key_prefix}/{s3_key}"
@@ -76,7 +155,7 @@ class S3(RedisModule):
             "aws_s3_bucket": self.aws_s3_bucket,
             "aws_s3_public": self.aws_s3_public,
             "s3_key": s3_key,
-            "image_filename": image_filename,
+            "filename": filename,
             "aws_access_key": self.aws_access_key,
             "aws_secret_key": self.aws_secret_key,
             "aws_region_name": self.aws_region_name,
@@ -85,83 +164,24 @@ class S3(RedisModule):
         try:
             queue = self.get_redis_queue()
             if queue:
-                queue.enqueue(self._upload, **arguments)
+                queue.enqueue(self.aws_s3_upload_function, **arguments)
             else:
-                self._upload(**arguments)
+                self.aws_s3_upload_function(**arguments)
 
-            result = {"s3_image_uri": f"s3://{self.aws_s3_bucket}/{s3_key}"}
+            result = {"s3_file_url": f"s3://{self.aws_s3_bucket}/{s3_key}"}
 
             if self.aws_s3_public:
                 result[
-                    "s3_image_public_uri"
+                    "s3_file_public_url"
                 ] = f"https://{self.aws_s3_bucket}.s3.amazonaws.com/{s3_key}"
 
             return result
 
         except Exception as e:
-            logging.exception("There was an error sending the notification email", e)
+            print(e)
+            logging.exception("There was an error sending the notification email")
             return None
 
-    def _upload(
-        self,
-        aws_s3_bucket,
-        aws_s3_public,
-        s3_key,
-        image_filename,
-        aws_access_key,
-        aws_secret_key,
-        aws_region_name,
-    ):
-        logging.info(
-            f'Uploading {image_filename} to bucket "{aws_s3_bucket}" and location {s3_key}...'
-        )
-        try:
-            session = boto3.Session(
-                aws_access_key_id=aws_access_key,
-                aws_secret_access_key=aws_secret_key,
-                region_name=aws_region_name,
-            )
-            client = session.client("s3")
-
-            with open(image_filename, "rb") as file:
-                buffer = file.read()
-
-            arguments = {"Bucket": aws_s3_bucket, "Key": s3_key, "Body": buffer}
-            content_type = "binary/octet-stream"
-
-            extension = Path(image_filename).suffix.lower()
-
-            if extension in ["jpg", "jpeg"]:
-                content_type = "image/jpeg"
-            elif extension == "png":
-                content_type = "image/png"
-
-            arguments["ContentType"] = content_type
-
-            if aws_s3_public:
-                arguments["ACL"] = "public-read"
-
-            client.put_object(**arguments)
-
-            s3_location = f"s3://{aws_s3_bucket}/{s3_key}"
-            logging.info(f"Uploaded {image_filename} to {s3_location}")
-
-            return s3_location
-
-        except EndpointConnectionError as e:
-            logging.exception(e)
-
-            # If we can't connect to the service to upload the image, let's
-            # raise a RuntimeError so that the process can be retried as part
-            # of the Redis queue.
-            raise RuntimeError(e)
-        except ClientError as e:
-            logging.exception(e)
-        except Exception as e:
-            logging.exception(e)
-
-        return None
-
     def _load_function(self, function_name: str):
         if function_name is None:
             return None
