@@ -107,7 +107,12 @@ class Pipeline(object):
 
             output = module.run(instance=instance)
             if output is not None:
-                instance.properties[module_name] = output
+                output_key = (
+                    f"{module_name}.{module.id}"
+                    if hasattr(module, "id") and module.id is not None
+                    else module_name
+                )
+                instance.properties[output_key] = output
 
             if output is not None and isinstance(output, dict):
                 self._create_output_artifact(
@@ -204,7 +209,7 @@ class Pipeline(object):
             for argument_id, argument in module_configuration.items():
                 if argument_id == "module":
                     continue
-                
+
                 arguments[argument_id] = argument
 
             arguments["redis_server"] = self.redis_server
